/*
 *  GUI/Sidebar/SearchFoldersSidebar.cs
 *  Copyright (C) 2006-2007 Marcin Krystian Krzywonos
 *  License: GNU/GPL version 2
 */


using System;
using System.Reflection;
using System.Collections;
using Mono.Unix;
using Gtk;
using Gdk;
using Incollector.GUI;
using Incollector.Engine;
using Incollector.Searching;


namespace Incollector.GUI {

	public class SearchFoldersSidebar : Gtk.VBox {
		
		protected enum ColsEnum {
			Icon         = 0,
			Title        = 1,
			SelectedIcon = 2,
			ID           = 3
		};
		
		
		
		protected TreeView   list;
		protected TreeIter   ActiveIter = new Gtk.TreeIter ();
		protected Gtk.Button AddBtn;
		protected new Gtk.Window ParentWindow;
		
		
		public delegate void UpdateActiveFolderHandler ();
		public event UpdateActiveFolderHandler ActiveFolderUpdated;
		
		
		public int SelectedSearchFolder {
			get {
				Gtk.TreeIter  itr;
				Gtk.TreeModel mod;
				
				if (list.Selection.GetSelected (out mod, out itr))
					return (int) (mod as ListStore).GetValue (itr, (int) ColsEnum.ID);
				else
					return -1;
			}
		}
		
		
		protected Gtk.Button RemoveBtn, EditBtn;
		
		
		public SearchFoldersSidebar (Gtk.Window ParentWindow)
		{
			this.ParentWindow = ParentWindow;
			
			SearchFolders.Added    += new SearchFolders.AddedHandler (on_SearchFolder_added);
			SearchFolders.Removed  += new SearchFolders.RemovedHandler (on_SearchFolder_removed);
			SearchFolders.Modified += new SearchFolders.ModifiedHandler (on_SearchFolder_modified);
			SearchFolders.Activated += new SearchFolders.ActivatedHandler (on_SearchFolder_activated);
			
			
			// creating treeview widget
			list = new Gtk.TreeView ();
			
			Gtk.ListStore store = new Gtk.ListStore (typeof (Gdk.Pixbuf), typeof (string), typeof (Gdk.Pixbuf), typeof (int));
			list.Model = store;
			list.EnableSearch = false;
			list.HeadersVisible = false;
			list.Selection.Changed += new EventHandler (on_list_selection_changed);
			list.RowActivated += new RowActivatedHandler (on_row_activated);
			
			CellRendererPixbuf rend = new CellRendererPixbuf ();
			rend.Xalign = 0;
			list.AppendColumn ("", rend, "pixbuf", 0);
			
			list.AppendColumn ("", new CellRendererText (), "markup", 1);
			
			CellRendererPixbuf rend2 = new CellRendererPixbuf ();
			rend2.Xalign = (float) 1.0;
			list.AppendColumn ("", rend2, "pixbuf", 2);
			
			
			// creating scrollbar widget
			Gtk.ScrolledWindow scroll = new Gtk.ScrolledWindow ();
			scroll.ShadowType = Gtk.ShadowType.In;
			scroll.SetPolicy (Gtk.PolicyType.Automatic, Gtk.PolicyType.Automatic); 
			scroll.Add (list);
			this.PackStart (scroll, true, true, 0);
			
			
			Gtk.HBox ButtonsBox = new Gtk.HBox ();
			ButtonsBox.Spacing = 6;
			ButtonsBox.BorderWidth = 0;
			this.PackStart (ButtonsBox, false, false, 0);
			
			AddBtn = new Gtk.Button ();
			AddBtn.Clicked += new EventHandler (on_AddBtn_clicked);
			AddBtn.Add (new Gtk.Image (Gtk.Stock.Add, Gtk.IconSize.Button));
			ButtonsBox.PackStart (AddBtn, false, false, 0);
			
			RemoveBtn = new Gtk.Button ();
			RemoveBtn.Clicked += new EventHandler (on_RemoveBtn_clicked);
			RemoveBtn.Add (new Gtk.Image (Gtk.Stock.Remove, Gtk.IconSize.Button));
			RemoveBtn.Sensitive = false;
			ButtonsBox.PackStart (RemoveBtn, false, false, 0);
			
			EditBtn = new Gtk.Button ();
			EditBtn.Clicked += new EventHandler (on_EditBtn_clicked);
			EditBtn.Add (new Gtk.Image (Gtk.Stock.Edit, Gtk.IconSize.Button));
			EditBtn.Sensitive = false;
			ButtonsBox.PackEnd (EditBtn, false, false, 0);
			
			
			this.Spacing = 6;
			this.ShowAll ();
			
			
			AddMainSearchFolder (Catalog.GetString ("All the entries"), Icons.GetPixbuf (Icons.SHOW_ALL_LIST), -1);
		}
		
		
		protected void on_SearchFolder_activated (SearchFolder folder)
		{
			this.ActivateFolder (folder.ID);
		}
		
		
		protected void on_list_selection_changed (object o, EventArgs args)
		{
			Gtk.TreeIter  itr;
			Gtk.TreeModel mod;
			
			RemoveBtn.Sensitive = EditBtn.Sensitive = 
				(list.Selection.GetSelected (out mod, out itr)) &&
				((int) (mod as ListStore).GetValue (itr, (int) ColsEnum.ID) != -1);
		}
		
		
		protected void on_SearchFolder_added (SearchFolder folder)
		{
			(list.Model as ListStore).AppendValues (new object [] {
					Icons.GetPixbuf (Icons.SEARCH_FOLDER_LIST),
					folder.Name, null, folder.ID});
		}
		
		
		protected void on_SearchFolder_removed (SearchFolder folder)
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = list.Model as ListStore;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				if (folder.ID == (int) store.GetValue (itr, (int) ColsEnum.ID)) {
					store.Remove (ref itr);
					return;
				}
		}
		
		
		protected void on_SearchFolder_modified (SearchFolder folder)
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = list.Model as ListStore;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				if (folder.ID == (int) store.GetValue (itr, (int) ColsEnum.ID)) {
					store.SetValue (itr, (int) ColsEnum.Title, folder.Name);
					
					if (store.IterIsValid (ActiveIter) && 
							(string.Compare (store.GetPath (itr).ToString (), store.GetPath (ActiveIter).ToString ()) == 0) &&
							(ActiveFolderUpdated != null))
						ActiveFolderUpdated ();
					
					return;
				}
		}
		
		
		protected void on_row_activated (object o, RowActivatedArgs args)
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = list.Model as ListStore;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				store.SetValue (itr, (int) ColsEnum.SelectedIcon, Icons.GetPixbuf ("empty.png"));
			
			TreeIter  iter;
			store.GetIter (out iter, args.Path);
			
			ActiveIter = iter;
			store.SetValue (ActiveIter, (int) ColsEnum.SelectedIcon,
							Icons.GetPixbuf (Icons.RIGHT_ARROW_LIST));
			
			new SearchFolder ((int) store.GetValue (iter, (int) ColsEnum.ID)).Activate ();
		}
		
		
		protected void on_AddBtn_clicked (object o, EventArgs args)
		{
			this.AddFolder ();
		}
		
		
		protected void on_RemoveBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeIter  itr;
			Gtk.TreeModel mod;
			
			if (list.Selection.GetSelected (out mod, out itr)) {
				this.SetParentSensitive (false);
				SearchFolder folder = new SearchFolder ((int) (mod as ListStore).GetValue (itr, (int) ColsEnum.ID));
				
				MessageDialog msg = new MessageDialog (this.ParentWindow, DialogFlags.DestroyWithParent,
							MessageType.Question, ButtonsType.YesNo,
							String.Format (Catalog.GetString ("Are you really sure you want to remove <i>{0}</i> search folder?"), folder.Name));
				
				if (msg.Run () == (int) ResponseType.Yes) {
					folder.Remove ();
					Searching.SearchFolders.Save ();
				}
				
				msg.Destroy ();
				this.SetParentSensitive (true);
			}
		}
		
		
		protected void on_EditBtn_clicked (object o, EventArgs args)
		{
			Gtk.TreeIter  itr;
			Gtk.TreeModel mod;
			
			if (list.Selection.GetSelected (out mod, out itr)) {
				this.SetParentSensitive (false);
				
				SearchFolder folder = new SearchFolder ((int) (mod as ListStore).GetValue (itr, (int) ColsEnum.ID));
				SearchFolderDialog dialog = new SearchFolderDialog (this.ParentWindow, folder);
				
				if (dialog.dialog.Run () == (int) ResponseType.Ok) {
					folder.Name = dialog.Name;
					folder.SearchOperator = dialog.ShowingEntriesOperator;
					folder.Queries = dialog.QueriesList;
					folder.Save ();
					Searching.SearchFolders.Save ();
				}
				
				dialog.dialog.Destroy ();
				this.SetParentSensitive (true);
			}
		}
		
		
		protected void AddMainSearchFolder (string title, Gdk.Pixbuf icon, int ID)
		{
			(list.Model as ListStore).AppendValues (new object [] {icon, title, null, ID});
		}
		
		
		public void AddFolder ()
		{
			SearchFolderDialog dialog = new SearchFolderDialog (this.ParentWindow);
			this.SetParentSensitive (false);
			
			if (dialog.dialog.Run () == (int) ResponseType.Ok) {
				Searching.SearchFolder folder = new Searching.SearchFolder ();
				folder.Name = dialog.Name;
				folder.SearchOperator = dialog.ShowingEntriesOperator;
				folder.Queries = dialog.QueriesList;
				folder.Save ();
				Searching.SearchFolders.Save ();
			}
			
			dialog.dialog.Destroy ();
			this.SetParentSensitive (true);
		}
		
		
		protected void ActivateFolder (int ID)
		{
			Gtk.TreeIter  itr;
			Gtk.ListStore store = list.Model as ListStore;
			
			for (bool tmp = store.GetIterFirst (out itr); tmp == true; tmp = store.IterNext (ref itr))
				if (ID == (int) store.GetValue (itr, (int) ColsEnum.ID)) {
					list.ActivateRow (store.GetPath (itr), list.Columns [1]);
					return;
				}
		}
		
		
		protected void SetParentSensitive (bool Sensitive)
		{
			this.ParentWindow.Sensitive = Sensitive;
		}
	}
}
